@@ -7,7 +7,7 @@
 /* Copyright Colas Nahaboo 1996, http://www.inria.fr/koala/colas. 
  * See http://www.inria.fr/koala/colas/browser-history
  */
-char *RCS_ID = "$Id: browser-history.c,v 2.4 1997/08/27 07:39:21 colas Exp $";
+char *RCS_ID = "$Id: browser-history.c,v 2.8 2000/07/25 15:46:26 colas Exp $";
 static char VERSION[8];
 /* Compile with:
    gcc -ansi -Wall browser-history.c -o browser-history -lXmu -lX11
@@ -39,7 +39,7 @@ void
 usage()
 {
     extern char *homedir, *logreldir;
-    P(E, "    browser-history v%s\n", VERSION);
+    P(E, " browser-history v%s\n", VERSION);
     P(E, "    See full docs at the URL:  http://www.inria.fr/koala/colas/browser-history\n");
     P(E, "    USAGE: browser-history [options]\n");
 
@@ -56,6 +56,36 @@ usage()
     P(E, "    In this file, empty lines or lines beginning by # are comments\n");
     P(E, "    This file is read once at startup, and re-read when receiving the signal 1\n");
 
+    P(E, " Options:\n");
+    P(E, "    -display display_name\n");
+    P(E, "         Specifies X display, otherwise contents of $DISPLAY is used\n");
+    P(E, "    -verbose\n");
+    P(E, "         outputs information on what it is doing. useful for debug.\n"); 
+    P(E, "    -Version\n"); 
+    P(E, "         prints version number and exit.\n"); 
+    P(E, "    -logdir directory\n"); 
+    P(E, "         which directory to store files into? defaults to ~/.browser-history\n"); 
+    P(E, "    -gzip gzip_filename\n"); 
+    P(E, "         the complete path to the gzip compressor. Defaults to gzip.\n"); 
+    P(E, "         E.g: -gzip /usr/gnu/bin/gzip\n"); 
+    P(E, "    -seconds delay\n"); 
+    P(E, "         if two entries are made are more than delay seconds apart, an\n");
+    P(E, "         horizontal rule will separate them, else just a simple line break.\n"); 
+    P(E, "         Defaults to one hour (3600).\n"); 
+    P(E, "    -replace\n"); 
+    P(E, "         If there is an already running browser-history on the display, kills\n");
+    P(E, "         it and take its place. Default is to replace it only if the version\n"); 
+    P(E, "         is older than ours.\n"); 
+    P(E, "    -noreplace\n"); 
+    P(E, "         If there is an already running browser-history on the display, aborts.\n");
+    P(E, "         Default is to replace it only if the version is older than ours.\n"); 
+    P(E, "    -kill\n"); 
+    P(E, "         If there is an already running browser-history on the display, kills\n");
+    P(E, "         it, then terminates immediately in all cases.\n"); 
+    P(E, "    -DontGrab\n"); 
+    P(E, "         Never Grab the X Server, which might cause deadlocks while debugging,\n");
+    P(E, "         when browser-history or gdb tries to print on the\n"); 
+    P(E, "         grabbed xterm or emacs.\n"); 
     exit(1);
 }
 
@@ -95,8 +125,9 @@ typedef struct _Browser_win {
 char *logreldir = ".browser-history";	/* relative to homedir */
 char *logfilename = "history-log.html";
 char *logfilebase = "history-log";
+char *headerfilename = "header.html";
 char *logfilepath;			/* expansed ~/logfilename */
-char *logfilebasepath;			/* expansed ~/logfilebase */
+char *headerfilepath;			/* expansed ~/headerfilename */
 char *excludefilename = "history-log.exclude";
 char *excludefilepath;			/* expansed ~/excludefilename */
 char *gzippath = "gzip";
@@ -132,6 +163,7 @@ Atom ARENA_LOCATION;
 Atom BROWSER_HISTORY_INFO;
 Atom BROWSER_HISTORY;
 Atom LAST_URL;
+Atom LAST_POS;
 Atom PROCESS;
 
 char *dayname[] = {
@@ -150,7 +182,7 @@ extern Window XmuClientWindow (Display *dpy, Window win);
 extern char *getenv(const char *);
 extern char *strdup(const char *);
 #define streq !strcmp
-extern char *strchr(const char *s, int c);
+/* extern char *strchr(const char *s, int c); */
 extern int atoi(char *s);
 extern int gethostname(char *name, size_t len);
 extern int system(char *command);
@@ -176,13 +208,13 @@ int terseXHandler(Display *dpy, XErrorEvent *error);
 int is_browser_window(Window win);
 int decode_ARENA_LOCATION(char *full_name, char **namep, char **urlp);
 void init_log_file(int first, FILE *fd);
-void add_log_entry(char *name, char *url, unsigned int win,
+long add_log_entry(char *name, char *url, unsigned int win,
 		   char *datestring, struct tm *times, int separator);
 void reinit();
 Window create_tag_window(int mode, int version, char *machine, int pid);
 char *date2string(struct tm *times);
 void update_tag_window(Window w, char *name, char *url, unsigned int win,
-		       char *datestring, int separator);
+		       char *datestring, int separator, long filepos);
 void init_excludefile();
 void check_daily_cleanup();
 void init_last_entry();
@@ -206,12 +238,13 @@ extern int sv_sprintfs(char *buffer, char *format, char *arg);
 ************************************ body *************************************
 \*****************************************************************************/
 
-void
+int
 main(int argc, char**argv)
 {
     initialize(argc, argv);
     detect_all_existing_browser_windows();
     mainloop();
+    return 0;
 }
 
 void
@@ -342,7 +375,7 @@ initialize(int argc, char**argv)
 
     /* expands file names wrt homedir (may have been changed by options) */
     logfilepath = concat_paths(homedir, logreldir, logfilename);
-    logfilebasepath = concat_paths(homedir, logreldir, logfilebase);
+    headerfilepath = concat_paths(homedir, logreldir, headerfilename);
     excludefilepath = concat_paths(homedir, logreldir, excludefilename);
     init_excludefile();
 
@@ -374,6 +407,7 @@ initialize(int argc, char**argv)
     BROWSER_HISTORY = XInternAtom(dpy, "BROWSER_HISTORY", False);
     BROWSER_HISTORY_INFO = XInternAtom(dpy, "BROWSER_HISTORY_INFO", False);
     LAST_URL = XInternAtom(dpy, "LAST_URL", False);
+    LAST_POS = XInternAtom(dpy, "LAST_POS", False);
     PROCESS = XInternAtom(dpy, "PROCESS", False);
 
     /* dont bother users with errors. this is a deamon */
@@ -405,8 +439,8 @@ parse_excluded_URLs()
 	    P(E, "reading %s\n", excludefilepath);
 	while (fgets(lineraw, 4000, fd)) {
 	    /* trim spaces ate beginning and end */
-	    for (line = lineraw; *line && isspace(*line); line++) ;
-	    for (end = line + strlen(line); end > line && isspace(*(end-1));
+	    for (line = lineraw; *line && isspace((int) *line); line++) ;
+	    for (end = line + strlen(line); end > line && isspace((int) *(end-1));
 		 end--) ;
 	    *end = '\0';
 	    if (line[0] != '\0' && line[0] != '#') {
@@ -486,20 +520,51 @@ init_excludefile()
 void
 init_log_file(int first, FILE *fd)
 {
-    fprintf(fd, "<head><title>Global history by browser-history</title></head>\n");
-    fprintf(fd, "<h1>Global history by browser-history</h1>\n");
-    fprintf(fd, "For more info, see the <a \n");
-    fprintf(fd, "href=\"http://www.inria.fr/koala/colas/browser-history\">\n");
-    fprintf(fd, "browser-history home page</a>\n");
-    fprintf(fd, "<h1>Past weeks</h1>\n");
-    if (first)
-	fprintf(fd, "None available. This is the first week of usage\n");
+    int i;
+    FILE *fh = fopen(headerfilepath, "r");
+    if (!fh) {				/* default header */
+	fprintf(fd, "<head><title>Global history by browser-history</title></head>\n");
+	fprintf(fd, "<h1>Global history by browser-history</h1>\n");
+	fprintf(fd, "For more info, see the <a \n");
+	fprintf(fd, "href=\"http://www.inria.fr/koala/colas/browser-history\">\n");
+	fprintf(fd, "browser-history home page</a>\n");
+	fprintf(fd, "<h1>This week</h1>\n");
+	fprintf(fd, "See at end of file, or jump directly to:\n<ul>\n");
+	for(i = 0; i < 7; i++) {
+	    fprintf(fd, "  <li><a href=\"#%s\">%s</a>\n", dayname[i], dayname[i]);
+	}
+	fprintf(fd, "</ul>\n");
+	fprintf(fd, "<h1>Past weeks</h1>\n"); /* end phrase default headers */
+    } else {				/* user-provided header */
+	char buffer[255];
+	int nc, inc;
+	/* copy header in log_file */
+	while ((nc = fread(buffer, 1, 255, fh)) != 0) {
+	    inc = fwrite(buffer, 1, nc, fd);
+	    if (inc != nc) {
+		P(E, "Warning: Invalid write (%i != %i) in %s!\n", 
+		  inc, nc, logfilepath);
+	    }
+	}
+	if (!feof(fh)) {
+	    P(E, "Warning: Invalid read in %s!\n", headerfilepath);
+	}
+	if (fclose(fh) == EOF) {
+	    P(E, "Warning: Can't close %s!\n", headerfilepath);
+	}
+    }
+    if (first) {
+        fprintf(fd, "<p>None available. This is the first week of usage</p>\n");
+    }
+    fprintf(fd, "<!-- EndOfHeader -->\n"); /* only present since v2.6 */
+    fflush(fd);
 }
 
-void
+long					/* current filepos in file, or -1 */
 add_log_entry(char *name, char *url, unsigned int win,
 	      char *datestring, struct tm *times, int separator)
 {
+    long filepos = -1;
     FILE *fd = fopen(logfilepath, "a");
     if (!fd) {			/* cannot open? try to make dir */
 	char logdir[1024];
@@ -509,7 +574,7 @@ add_log_entry(char *name, char *url, unsigned int win,
     }
     if (!fd) {				/* cannot do anything */
 	P(E, "Warning: could not open %s!\n", logfilepath);
-	return;
+	return filepos;
     }
     if (ftell(fd) == 0) {		/* init file if empty */
 	init_log_file(1, fd);
@@ -521,10 +586,13 @@ add_log_entry(char *name, char *url, unsigned int win,
 		"%s <b>%s</b> <a href=\"%s\">%s</a> %s <small>0x%x</small>\n",
 		separator ? "<hr>" : "<br>",
 		name, url, url, datestring, win);
-
+	fflush(fd);
+	filepos = ftell(fd);
 	fclose(fd);
-	update_tag_window(tag_window, name, url, win, datestring, separator);
+	update_tag_window(tag_window, name, url, win, datestring, 
+			  separator, filepos);
     }
+    return filepos;
 }
 
 FILE *					/* can be modified... */
@@ -544,9 +612,9 @@ add_log_day(FILE *fd, struct tm *times)
 		P(E, "New week! Starting new log file, archiving previous\n");
 	    sprintf(command, "cd %s; if test -f %s; then mv %s %s-%02d-%02d-%02d.html; %s -9 %s-%02d-%02d-%02d.html; fi",
 		    dir, logfilename, logfilename, logfilebase,
-		    times->tm_year, times->tm_mon+1, times->tm_mday, gzippath,
-		    logfilebase, times->tm_year, times->tm_mon+1, 
-		    times->tm_mday);
+		    times->tm_year+1900, times->tm_mon+1, times->tm_mday,
+		    gzippath, logfilebase, times->tm_year+1900,
+		    times->tm_mon+1, times->tm_mday);
 	    system(command);
 	    fd = fopen(logfilepath, "a");
 	    if (!fd) return 0;
@@ -557,8 +625,8 @@ add_log_day(FILE *fd, struct tm *times)
 	     * fix courtesy of canon@ecoledoc.ibp.fr (Hubert CANON)
 	     */
 	    fclose(fd);			
-	    sprintf(command, "cd %s; for i in `ls -1r %s-[0-9]*.html*`; do echo \"<a href=\\\"file:$i\\\">`basename $i .html.gz`</a><br>\">>%s; done; echo '<hr>' >> %s",
-		    dir, logfilebasepath, logfilename, logfilename);
+	    sprintf(command, "cd %s; for i in `ls -1r %s-[0-9]*.html*`; do echo \"<a href=\\\"$i\\\">`basename $i .html.gz`</a><br>\">>%s; done; echo '<hr>' >> %s",
+		    dir, logfilebase, logfilename, logfilename);
 	    system(command);
 	    Free(dir);
  	    fd = fopen(logfilepath, "a"); 
@@ -566,9 +634,10 @@ add_log_day(FILE *fd, struct tm *times)
 	}
 	if (verbose)
 	    P(E, "New day! adding day chapter in log file\n");
-	fprintf(fd, "<hr><p><h1>%s %d %s %d</h1>\n",
-		dayname[times->tm_wday], times->tm_mday, 
-		monthname[times->tm_mon], times->tm_year);
+	/* warning: previous to v2.6, there was no <a> in day names */
+	fprintf(fd, "<hr><p><h1><a name=\"%s\">%s %d %s %d</a></h1>\n",
+		dayname[times->tm_wday], dayname[times->tm_wday], 
+		times->tm_mday, monthname[times->tm_mon], times->tm_year+1900);
     } /* same day, do nothing */
     return fd;
 }
@@ -578,7 +647,7 @@ date2string(struct tm *times)
 {
     static char datestring[40];
     sprintf(datestring, "%d/%02d/%02d-%02d:%02d:%02d",
-	    times->tm_year, times->tm_mon + 1, times->tm_mday, 
+	    times->tm_year+1900, times->tm_mon + 1, times->tm_mday, 
 	    times->tm_hour, times->tm_min, times->tm_sec);
     return datestring;
 }
@@ -759,14 +828,23 @@ add_URL(int number)
     time_t seconds;
     struct tm *times;
     char *name, *url;
-    char *full_name = 0;
+    char *full_name = 0, *ns_name = 0;
     Window win;
 
     if (number < 0) return;
     win = browser_windows[number].win;
     
     if (browser_windows[number].browser_type == BT_NETSCAPE) { /* netscape */
-	name = get_prop(win, WM_NAME, 10);	/* name, skips Netscape:  */
+	ns_name = get_prop(win, WM_NAME, 0);	/* name */
+	if (!strncmp("Netscape:", ns_name, 9)) { /* skip leading Netscape: */
+	    if (ns_name[9] == ' ') {
+		name = ns_name + 10;	/* skip also blank if there */
+	    } else {
+		name = ns_name + 9;
+	    }
+	} else {
+	    name = ns_name;
+	}
 	url = get_prop(win, _MOZILLA_URL, 0);
     } else if (browser_windows[number].browser_type == BT_ARENA) { /* arena */
 	if (nilstring == (full_name = get_prop(win, ARENA_LOCATION, 0)))
@@ -813,7 +891,7 @@ add_URL(int number)
 	last_time = seconds;
     }
     if (browser_windows[number].browser_type == BT_NETSCAPE)
-	xfree(name, 10), xfree(url, 0);
+	xfree(ns_name, 0), xfree(url, 0);
     else /* arena, amaya, generic */
 	xfree(full_name, 0);
 }
@@ -831,6 +909,10 @@ add_URL(int number)
  *   URL=url_text\0TITLE=page_title\0DATE=date_string\0BROWSER_WINDOW_ID=id\0
  *   This property will be updated on each new entry in the log file.
  *   at start, it exists but is empty (length = 0)
+ * - LAST_POS (since v2.7) a text property containing the index (ascii number)
+ *   of the last updated position in the current log file. This allows 
+ *   external processes to read the log file and stop at the logical end
+ *   in case they read just while we are appending. Can be -1 if not defined
  * - PROCESS     a text property, concatenation of null terminated strings
  *   prefixed by a tag=, giving a description of the running process:
  *   version    a number (card32) made by 1000*major+minor: v3.4 => 3004
@@ -932,6 +1014,8 @@ OK:
 		    32, PropModeReplace, (unsigned char *) &w, 1);
     XChangeProperty(dpy, w, LAST_URL, XA_STRING,
 		    8, PropModeReplace, (unsigned char *) "", 0);
+    XChangeProperty(dpy, w, LAST_POS, XA_STRING,
+		    8, PropModeReplace, (unsigned char *) "", 0);
     len += sv_sprintfd(string, "VERSION=%d", version) + 1; /* must be 1st */
     len += sv_sprintfs(string + len, "DATE=%s", datestring) + 1;
     len += sv_sprintfs(string + len, "MACHINE=%s", machine) + 1;
@@ -945,7 +1029,7 @@ OK:
 
 void
 update_tag_window(Window w, char *name, char *url, unsigned int win,
-		  char *datestring, int separator)
+		  char *datestring, int separator, long filepos)
 {
     char string[8000];
     int len = 0;
@@ -955,6 +1039,11 @@ update_tag_window(Window w, char *name, char *url, unsigned int win,
     len += sv_sprintfd(string + len, "BROWSER_WINDOW_ID=%d", win) + 1;
     XChangeProperty(dpy, w, LAST_URL, XA_STRING, 8, PropModeReplace, 
 		    (unsigned char *) string, len);
+
+    len = 0;
+    len += sv_sprintfs(string, "%ld", filepos) + 1;
+    XChangeProperty(dpy, w, LAST_POS, XA_STRING, 8, PropModeReplace, 
+		    (unsigned char *) string, len);
 }
 
 /*****************************************************************************\
@@ -1080,7 +1169,7 @@ void
 init_last_entry()
 {
     FILE *fd;
-    /* on failure to find date, set time to 70/01/01 */
+    /* on failure to find date, set time to 1970/01/01 */
     last_times.tm_year = 70;
     last_times.tm_mon = 0;
     last_times.tm_mday = 1;
@@ -1097,7 +1186,10 @@ init_last_entry()
 	line = strstr(buffer, "</a>");	/* look for occurence of </a> */
 	if (line && 6 == sscanf(line, "</a> %d/%02d/%02d-%02d:%02d:%02d",
 				&y,&m,&d,&h,&mm,&s)) {
-	    last_times.tm_year = y;
+	    if (y < 100)			/* Fix Y2K bug */
+	      last_times.tm_year = y;		/* Assume before 2000 */
+	    else
+	      last_times.tm_year = y - 1900;	/* y contains full year */
 	    last_times.tm_mon = m - 1;
 	    last_times.tm_mday = d;
 	    last_times.tm_hour = h;
@@ -1112,7 +1204,7 @@ init_last_entry()
 	last_time = mktime(&last_times);
     if (verbose)
 	P(E, "Last entry was at: %d/%02d/%02d-%02d:%02d:%02d (day of week: %d (%s), day of year: %d)\n",
-	  last_times.tm_year, last_times.tm_mon + 1, last_times.tm_mday, 
+	  last_times.tm_year+1900, last_times.tm_mon + 1, last_times.tm_mday, 
 	  last_times.tm_hour, last_times.tm_min, last_times.tm_sec,
 	  last_times.tm_wday, dayname[last_times.tm_wday], last_times.tm_yday);
 }
